==> apache-doris-4.0.2-rc02-src.tar.gz: apache-doris-4.0.2-rc02-src/fe/fe-core/src/test/java/org/apache/doris/catalog/RefreshTableTest.java <==
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package org.apache.doris.catalog;

import org.apache.doris.analysis.UserIdentity;
import org.apache.doris.common.AnalysisException;
import org.apache.doris.common.ExceptionChecker;
import org.apache.doris.common.FeConstants;
import org.apache.doris.datasource.CatalogIf;
import org.apache.doris.datasource.infoschema.ExternalInfoSchemaDatabase;
import org.apache.doris.datasource.infoschema.ExternalInfoSchemaTable;
import org.apache.doris.datasource.infoschema.ExternalMysqlDatabase;
import org.apache.doris.datasource.infoschema.ExternalMysqlTable;
import org.apache.doris.datasource.test.TestExternalCatalog;
import org.apache.doris.datasource.test.TestExternalTable;
import org.apache.doris.mysql.privilege.Auth;
import org.apache.doris.nereids.parser.NereidsParser;
import org.apache.doris.nereids.trees.plans.commands.CreateCatalogCommand;
import org.apache.doris.nereids.trees.plans.commands.CreateUserCommand;
import org.apache.doris.nereids.trees.plans.commands.DropCatalogCommand;
import org.apache.doris.nereids.trees.plans.commands.GrantTablePrivilegeCommand;
import org.apache.doris.nereids.trees.plans.commands.info.TableNameInfo;
import org.apache.doris.nereids.trees.plans.commands.refresh.RefreshTableCommand;
import org.apache.doris.nereids.trees.plans.logical.LogicalPlan;
import org.apache.doris.qe.ConnectContext;
import org.apache.doris.qe.StmtExecutor;
import org.apache.doris.utframe.TestWithFeService;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import java.util.List;
import java.util.Map;

public class RefreshTableTest extends TestWithFeService {
    private static Env env;
    private ConnectContext rootCtx;

    @Override
    protected void runBeforeAll() throws Exception {
        FeConstants.runningUnitTest = true;
        rootCtx = createDefaultCtx();
        env = Env.getCurrentEnv();
        // 1. create test catalog
        String createStmt = "create catalog test1 properties(\n"
                + "    \"type\" = \"test\",\n"
                + "    \"catalog_provider.class\" "
                + "= \"org.apache.doris.catalog.RefreshTableTest$RefreshTableProvider\"\n"
                + ");";

        NereidsParser nereidsParser = new NereidsParser();
        LogicalPlan logicalPlan = nereidsParser.parseSingle(createStmt);
        if (logicalPlan instanceof CreateCatalogCommand) {
            ((CreateCatalogCommand) logicalPlan).run(rootCtx, null);
        }
    }

    @Override
    protected void runAfterAll() throws Exception {
        super.runAfterAll();
        rootCtx.setThreadLocalInfo();
        NereidsParser nereidsParser = new NereidsParser();
        LogicalPlan logicalPlan = nereidsParser.parseSingle("drop catalog test1");
        if (logicalPlan instanceof DropCatalogCommand) {
            ((DropCatalogCommand) logicalPlan).run(rootCtx, null);
        }
    }

    @Test
    public void testRefreshTable() throws Exception {
        CatalogIf test1 = env.getCatalogMgr().getCatalog("test1");
        TestExternalTable table = (TestExternalTable) test1.getDbNullable("db1").getTable("tbl11").get();
        Assertions.assertFalse(table.isObjectCreated());
        long l1 = table.getSchemaUpdateTime();
        Assertions.assertTrue(l1 == 0);
        table.makeSureInitialized();
        Assertions.assertTrue(table.isObjectCreated());
        long l2 = table.getSchemaUpdateTime();
        Assertions.assertTrue(l2 == l1);
        TableNameInfo tableNameInfo = new TableNameInfo("test1", "db1", "tbl11");
        try {
            Env.getCurrentEnv().getRefreshManager()
                    .handleRefreshTable(tableNameInfo.getCtl(), tableNameInfo.getDb(), tableNameInfo.getTbl(), false);
        } catch (Exception e) {
            // Do nothing
        }
        Assertions.assertFalse(table.isObjectCreated());
        long l3 = table.getSchemaUpdateTime();
        Assertions.assertTrue(l3 == l2);
        table.getFullSchema();
        // only table.getFullSchema() can change table.lastUpdateTime
        long l4 = table.getSchemaUpdateTime();
        Assertions.assertTrue(l4 > l3);
        // updateTime is equal to schema update time as default
        long l5 = table.getUpdateTime();
        Assertions.assertTrue(l5 == l4);

        // external info schema db
        ExternalInfoSchemaDatabase infoDb = (ExternalInfoSchemaDatabase) test1.getDbNullable(InfoSchemaDb.DATABASE_NAME);
        Assertions.assertNotNull(infoDb);
        for (String tblName : SchemaTable.TABLE_MAP.keySet()) {
            ExternalInfoSchemaTable infoTbl = (ExternalInfoSchemaTable) infoDb.getTableNullable(tblName);
            Assertions.assertNotNull(infoTbl);
            List<Column> schema = infoTbl.getFullSchema();
            Assertions.assertEquals(SchemaTable.TABLE_MAP.get(tblName).getColumns().size(), schema.size());
        }
        // external mysql db
        ExternalMysqlDatabase mysqlDb = (ExternalMysqlDatabase) test1.getDbNullable(MysqlDb.DATABASE_NAME);
        Assertions.assertNotNull(mysqlDb);
        for (String tblName : MysqlDBTable.TABLE_MAP.keySet()) {
            ExternalMysqlTable mysqlTbl = (ExternalMysqlTable) mysqlDb.getTableNullable(tblName);
            Assertions.assertNotNull(mysqlTbl);
            List<Column> schema = mysqlTbl.getFullSchema();
            Assertions.assertEquals(MysqlDBTable.TABLE_MAP.get(tblName).getColumns().size(), schema.size());
        }
    }

    @Test
    public void testRefreshPriv() throws Exception {
        Auth auth = Env.getCurrentEnv().getAuth();
        // create user1
        NereidsParser nereidsParser = new NereidsParser();
        String sql = "create user 'user1'@'%' identified by 'pwd1';";
        LogicalPlan parsed = nereidsParser.parseSingle(sql);
        StmtExecutor stmtExecutor = new StmtExecutor(rootCtx, sql);
        if (parsed instanceof CreateUserCommand) {
            ((CreateUserCommand) parsed).run(rootCtx, stmtExecutor);
        }

        // mock login user1
        UserIdentity user1 = new UserIdentity("user1", "%");
        user1.analyze();
        ConnectContext user1Ctx = createCtx(user1, "127.0.0.1");
        RefreshTableCommand command = (RefreshTableCommand) parseStmt(
                "refresh table test1.db1.tbl11", user1Ctx);
        ExceptionChecker.expectThrowsWithMsg(AnalysisException.class,
                "Access denied",
                () -> command.run(rootCtx, stmtExecutor));
        ConnectContext.remove();

        // add drop priv to user1
        rootCtx.setThreadLocalInfo();
        LogicalPlan logicalPlan1 = nereidsParser.parseSingle("grant drop_priv on test1.db1.tbl11 to 'user1'@'%';");
        Assertions.assertTrue(logicalPlan1 instanceof GrantTablePrivilegeCommand);
        GrantTablePrivilegeCommand command1 = (GrantTablePrivilegeCommand) logicalPlan1;
        command1.validate();
        auth.grantTablePrivilegeCommand(command1);
        ConnectContext.remove();

        // user1 can do refresh table
        user1Ctx.setThreadLocalInfo();
        RefreshTableCommand command2 = (RefreshTableCommand) parseStmt("refresh table test1.db1.tbl11", user1Ctx);
        ExceptionChecker.expectThrowsNoException(
                () -> command2.run(rootCtx, stmtExecutor));
    }

    public static class RefreshTableProvider implements TestExternalCatalog.TestCatalogProvider {
        public static final Map<String, Map<String, List<Column>>> MOCKED_META;

        static {
            MOCKED_META = Maps.newHashMap();
            Map<String, List<Column>> tblSchemaMap1 = Maps.newHashMap();
            // db1
            tblSchemaMap1.put("tbl11", Lists.newArrayList(
                new Column("a11", PrimitiveType.BIGINT),
                new Column("a12", PrimitiveType.STRING),
                new Column("a13", PrimitiveType.FLOAT)));
            tblSchemaMap1.put("tbl12", Lists.newArrayList(
                new Column("b21", PrimitiveType.BIGINT),
                new Column("b22", PrimitiveType.STRING),
                new Column("b23", PrimitiveType.FLOAT)));
            MOCKED_META.put("db1", tblSchemaMap1);
            // db2
            Map<String, List<Column>> tblSchemaMap2 = Maps.newHashMap();
            tblSchemaMap2.put("tbl21", Lists.newArrayList(
                new Column("c11", PrimitiveType.BIGINT),
                new Column("c12", PrimitiveType.STRING),
                new Column("c13", PrimitiveType.FLOAT)));
            MOCKED_META.put("db2", tblSchemaMap2);
        }

        @Override
        public Map<String, Map<String, List<Column>>> getMetadata() {
            return MOCKED_META;
        }
    }
}

==> apache-doris-4.0.3-rc03-src.tar.gz: apache-doris-4.0.3-rc03-src/fe/fe-core/src/test/java/org/apache/doris/catalog/RefreshTableTest.java <==
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package org.apache.doris.catalog;

import org.apache.doris.analysis.UserIdentity;
import org.apache.doris.common.AnalysisException;
import org.apache.doris.common.ExceptionChecker;
import org.apache.doris.common.FeConstants;
import org.apache.doris.datasource.CatalogIf;
import org.apache.doris.datasource.infoschema.ExternalInfoSchemaDatabase;
import org.apache.doris.datasource.infoschema.ExternalInfoSchemaTable;
import org.apache.doris.datasource.infoschema.ExternalMysqlDatabase;
import org.apache.doris.datasource.infoschema.ExternalMysqlTable;
import org.apache.doris.datasource.test.TestExternalCatalog;
import org.apache.doris.datasource.test.TestExternalTable;
import org.apache.doris.mysql.privilege.Auth;
import org.apache.doris.nereids.parser.NereidsParser;
import org.apache.doris.nereids.trees.plans.commands.CreateCatalogCommand;
import org.apache.doris.nereids.trees.plans.commands.CreateUserCommand;
import org.apache.doris.nereids.trees.plans.commands.DropCatalogCommand;
import org.apache.doris.nereids.trees.plans.commands.GrantTablePrivilegeCommand;
import org.apache.doris.nereids.trees.plans.commands.info.TableNameInfo;
import org.apache.doris.nereids.trees.plans.commands.refresh.RefreshTableCommand;
import org.apache.doris.nereids.trees.plans.logical.LogicalPlan;
import org.apache.doris.qe.ConnectContext;
import org.apache.doris.qe.StmtExecutor;
import org.apache.doris.utframe.TestWithFeService;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import java.util.List;
import java.util.Map;

public class RefreshTableTest extends TestWithFeService {
    private static Env env;
    private ConnectContext rootCtx;

    @Override
    protected void runBeforeAll() throws Exception {
        FeConstants.runningUnitTest = true;
        rootCtx = createDefaultCtx();
        env = Env.getCurrentEnv();
        // 1. create test catalog
        String createStmt = "create catalog test1 properties(\n"
                + "    \"type\" = \"test\",\n"
                + "    \"catalog_provider.class\" "
                + "= \"org.apache.doris.catalog.RefreshTableTest$RefreshTableProvider\"\n"
                + ");";

        NereidsParser nereidsParser = new NereidsParser();
        LogicalPlan logicalPlan = nereidsParser.parseSingle(createStmt);
        if (logicalPlan instanceof CreateCatalogCommand) {
            ((CreateCatalogCommand) logicalPlan).run(rootCtx, null);
        }
    }

    @Override
    protected void runAfterAll() throws Exception {
        super.runAfterAll();
        rootCtx.setThreadLocalInfo();
        NereidsParser nereidsParser = new NereidsParser();
        LogicalPlan logicalPlan = nereidsParser.parseSingle("drop catalog test1");
        if (logicalPlan instanceof DropCatalogCommand) {
            ((DropCatalogCommand) logicalPlan).run(rootCtx, null);
        }
    }

    @Test
    public void testRefreshTable() throws Exception {
        CatalogIf test1 = env.getCatalogMgr().getCatalog("test1");
        TestExternalTable table = (TestExternalTable) test1.getDbNullable("db1").getTable("tbl11").get();
        Assertions.assertFalse(table.isObjectCreated());
        long l1 = table.getUpdateTime();
        // Assertions.assertEquals(0, l1);
        table.makeSureInitialized();
        Assertions.assertTrue(table.isObjectCreated());
        long l2 = table.getUpdateTime();
        Assertions.assertTrue(l2 >= l1);
        TableNameInfo tableNameInfo = new TableNameInfo("test1", "db1", "tbl11");
        try {
            Env.getCurrentEnv().getRefreshManager()
                    .handleRefreshTable(tableNameInfo.getCtl(), tableNameInfo.getDb(), tableNameInfo.getTbl(), false);
        } catch (Exception e) {
            // Do nothing
        }
        Assertions.assertFalse(table.isObjectCreated());
        long l3 = table.getUpdateTime();
        Assertions.assertTrue(l3 >= l2);
        table.getFullSchema();
        // only table.getFullSchema() can change table.lastUpdateTime
        long l4 = table.getUpdateTime();
        Assertions.assertTrue(l4 > l3);
        // updateTime is equal to schema update time as default
        long l5 = table.getUpdateTime();
        Assertions.assertTrue(l5 >= l4);

        // external info schema db
        ExternalInfoSchemaDatabase infoDb = (ExternalInfoSchemaDatabase) test1.getDbNullable(InfoSchemaDb.DATABASE_NAME);
        Assertions.assertNotNull(infoDb);
        for (String tblName : SchemaTable.TABLE_MAP.keySet()) {
            ExternalInfoSchemaTable infoTbl = (ExternalInfoSchemaTable) infoDb.getTableNullable(tblName);
            Assertions.assertNotNull(infoTbl);
            List<Column> schema = infoTbl.getFullSchema();
            Assertions.assertEquals(SchemaTable.TABLE_MAP.get(tblName).getColumns().size(), schema.size());
        }
        // external mysql db
        ExternalMysqlDatabase mysqlDb = (ExternalMysqlDatabase) test1.getDbNullable(MysqlDb.DATABASE_NAME);
        Assertions.assertNotNull(mysqlDb);
        for (String tblName : MysqlDBTable.TABLE_MAP.keySet()) {
            ExternalMysqlTable mysqlTbl = (ExternalMysqlTable) mysqlDb.getTableNullable(tblName);
            Assertions.assertNotNull(mysqlTbl);
            List<Column> schema = mysqlTbl.getFullSchema();
            Assertions.assertEquals(MysqlDBTable.TABLE_MAP.get(tblName).getColumns().size(), schema.size());
        }
    }

    @Test
    public void testRefreshPriv() throws Exception {
        Auth auth = Env.getCurrentEnv().getAuth();
        // create user1
        NereidsParser nereidsParser = new NereidsParser();
        String sql = "create user 'user1'@'%' identified by 'pwd1';";
        LogicalPlan parsed = nereidsParser.parseSingle(sql);
        StmtExecutor stmtExecutor = new StmtExecutor(rootCtx, sql);
        if (parsed instanceof CreateUserCommand) {
            ((CreateUserCommand) parsed).run(rootCtx, stmtExecutor);
        }

        // mock login user1
        UserIdentity user1 = new UserIdentity("user1", "%");
        user1.analyze();
        ConnectContext user1Ctx = createCtx(user1, "127.0.0.1");
        RefreshTableCommand command = (RefreshTableCommand) parseStmt(
                "refresh table test1.db1.tbl11", user1Ctx);
        ExceptionChecker.expectThrowsWithMsg(AnalysisException.class,
                "Access denied",
                () -> command.run(rootCtx, stmtExecutor));
        ConnectContext.remove();

        // add drop priv to user1
        rootCtx.setThreadLocalInfo();
        LogicalPlan logicalPlan1 = nereidsParser.parseSingle("grant drop_priv on test1.db1.tbl11 to 'user1'@'%';");
        Assertions.assertTrue(logicalPlan1 instanceof GrantTablePrivilegeCommand);
        GrantTablePrivilegeCommand command1 = (GrantTablePrivilegeCommand) logicalPlan1;
        command1.validate();
        auth.grantTablePrivilegeCommand(command1);
        ConnectContext.remove();

        // user1 can do refresh table
        user1Ctx.setThreadLocalInfo();
        RefreshTableCommand command2 = (RefreshTableCommand) parseStmt("refresh table test1.db1.tbl11", user1Ctx);
        ExceptionChecker.expectThrowsNoException(
                () -> command2.run(rootCtx, stmtExecutor));
    }

    public static class RefreshTableProvider implements TestExternalCatalog.TestCatalogProvider {
        public static final Map<String, Map<String, List<Column>>> MOCKED_META;

        static {
            MOCKED_META = Maps.newHashMap();
            Map<String, List<Column>> tblSchemaMap1 = Maps.newHashMap();
            // db1
            tblSchemaMap1.put("tbl11", Lists.newArrayList(
                new Column("a11", PrimitiveType.BIGINT),
                new Column("a12", PrimitiveType.STRING),
                new Column("a13", PrimitiveType.FLOAT)));
            tblSchemaMap1.put("tbl12", Lists.newArrayList(
                new Column("b21", PrimitiveType.BIGINT),
                new Column("b22", PrimitiveType.STRING),
                new Column("b23", PrimitiveType.FLOAT)));
            MOCKED_META.put("db1", tblSchemaMap1);
            // db2
            Map<String, List<Column>> tblSchemaMap2 = Maps.newHashMap();
            tblSchemaMap2.put("tbl21", Lists.newArrayList(
                new Column("c11", PrimitiveType.BIGINT),
                new Column("c12", PrimitiveType.STRING),
                new Column("c13", PrimitiveType.FLOAT)));
            MOCKED_META.put("db2", tblSchemaMap2);
        }

        @Override
        public Map<String, Map<String, List<Column>>> getMetadata() {
            return MOCKED_META;
        }
    }
}
